#include <iostream>         //  cout, endl
#include <cmath>            //  fabs()
#include <limits>           //  numeric_limits<double>::max()
using namespace std;


//                       Funktionen aus Aufgabe 20

//    Riemann-Integration
double riemann( double (* const func)(double), const double &a, const double &b,
		const int n)
{
  double s = 0.0,       //  Summe
         h = (b-a)/n;   //  Intervalllaenge

  for (int j=1; j<=n; ++j)
  {
    s += func(a+j*h)*h;
  }

  return s;
}


//    Differenzieren
double derivative( double (* const func)(double), const double &x, const double &h)
{
  return ( func(x+h) - func(x-h) )/2/h;
}


//                       Funktionen aus Aufgabe 21


//    Riemann-Integration (hier klappt Overloading da 4. Parameter anders)
double riemann( double (* const func)(double), const double &a, const double &b,
		const double eps)
{
 int    n = 5;        // Anzahl der Intervalle im Integrationsbereich [a,b]
 double r_old, r_new;

 //    Verdoppele die Anzahl der Intervalle, bis die geforderte Genauigkeit erreicht ist
 r_new = riemann(func, a, b, n);
 do
 {
   r_old = r_new;
   n = 2*n;
   r_new = riemann(func, a, b, n);
 }
 while ( fabs(r_new-r_old) > eps );

//  cout << " Abbruch bei  n = " << n << endl;

 return  r_new;
}

//                       Funktionen aus Aufgabe 21
//    Differenzieren (andere Funktionsname, da alle Parameter gleich sind
//                    und die Parameterliste nicht umgestellt wurde)
double derivative2( double (* const func)(double), const double &x, const double eps)
{
 double h = 0.1;       // Schrittweite fuer numerische Differentiation

 double r_old, r_new;

 //    Halbiere die Schrittweite h, bis die geforderte Genauigkeit erreicht ist
 r_new = derivative(func, x, h);
 do
 {
   r_old = r_new;
   h = h/2;
//    cout << " h = " << h << "  " << r_old << "  " << x << endl;
   r_new = derivative(func, x, h);
 }                                       // Maschinen-Epsilon beachten!!
 while ( fabs(r_new-r_old) > eps &&  h > 8*x*numeric_limits<double>::epsilon() );

 cout << " Abbruch bei  h = " << h << endl;

 return  r_new;
}

//    Newton solver fuer func(x) mit x aus [a,b]
//     auf eine Genauigkeit von |x_new-x_old| < eps
//    ACHTUNG: Hier fehlt der Test, ob x im Intervall [a,b] bleibt (kontraktive Abbildung)
double newton_diff_approx
 ( double (* const func)(double), const double &a, const double &b, const double eps=1e-4)
{
  const int MAX_ITER=100;
        int iter=0;
  const double ABBRUCH_X = -numeric_limits<double>::max();  // darauf kann man spaeter testen
  double x_old = ABBRUCH_X, x_new = (a+b)/2.0, fx, fx_1;

  while ( fabs(x_new-x_old) >= eps && iter <= MAX_ITER )
  {
    ++iter;
    x_old = x_new;
    fx    = func(x_old);
    fx_1  = derivative2(func, x_old, eps);
//     cout << fx << endl;
//     cout << fx_1 << endl<< endl;

    if ( fabs(fx_1)>eps )
    {
       x_new = x_old - fx/fx_1;
    }
    else                  //  func'(x) wird zu klein
    {
       x_new = x_old = ABBRUCH_X;
    }
    cout << "x_iter : " << x_new << endl;
  }

  if ( x_new == ABBRUCH_X )
  {
    cout << "\n f'(x) zu klein\n\n";
  }
  if ( iter > MAX_ITER)
  {
    cout << "\n zu viele Iterationen\n\n";
  }

  cout.precision(16);
  cout << "\n Newton-Iterationen: " << iter
       << "  bei Error  " << fabs(x_new-x_old) << endl;

  return x_new;

}

