//      SS07, Aufg. 23
//    Compilieren+Linken
//         g++ -Wall -pedantic bsp_23.cpp
//    Dokumentation
//         kdoc -p -d bsp_23 bsp_23.cpp
//         kdoc -p -d tex_bsp_23 -f latex bsp_23.cpp
#include <iostream>
#include <iomanip>             // setw
#include <cassert>             // assert
#include <cmath>               // round
using namespace std;

//  ####################################################################
//              Deklaration der Funktionen (gehoert in das Headerfile)
//  ####################################################################

/**
     Calculates the number of coins for each value such that
     the price is represented.
     @param	price	given price to be represented
     @param	coins	resulting number of different coins
     @param	values  decending currency values of the different coins
     @param	n	given number of different coins
*/
void num_coins(const int& price, int coins[], const int values[], const int n);

/**
     Prints the number of coins for each value such that
     the price is represented.
     @param	coins	number of different coins
     @param	values  decending currency values of the different coins
     @param	n	number of different coins
*/
void print_coins(const int coins[], const int values[], const int n);

/**
     Prints the number of coins for each value such that
     the price is represented.
     @param	coins	number of different coins
     @param	weights	weights of the different coins
     @param	n	number of different coins
     @return	accumulated weight of the coins in the purse
*/
float weight_coins(const int coins[], const float weights[], const int n);

//  ####################################################################
//             Hauptprogramm
//  ####################################################################

int main(const int argc, const char* argv[])
{
 float kp, gm;   // Kaufpreis und Geldmenge des Kunden
 cout << "Kaufpreis = "; cin >> kp;
 cout << "Geldmenge = "; cin >> gm;

 //  Umrechnung des Geldbetrages in Cent
 const int preis = static_cast<int>(round(kp*100)),      // Preis
           geld  = static_cast<int>(round(gm*100));      // Geld
 //  Muenzwerte in Cent
 const int wert[] = {200, 100, 50, 20, 10, 5, 2, 1};
 //  Anzahl der verschiedenen Muenzwerte
 const int nwerte = sizeof(wert)/sizeof(wert[0]);

 //  Muenzgewichte in Gramm
 //  Quelle:  http://www.austrian-mint.com/allgemeineEuroInformation?l=de
 const float gewicht[nwerte] = {8.50, 7.50, 7.80, 5.74, 4.10, 3.92, 3.06, 2.30};

 int muenzen[nwerte]={};    // mit 0 initialisieren

 num_coins( geld-preis, muenzen, wert, nwerte );
 print_coins( muenzen, wert, nwerte );

 double masse = weight_coins(muenzen, gewicht, nwerte);

 cout << endl;
 cout.setf(ios::fixed, ios::floatfield);
 cout.precision(2);
 cout << "Preis : " << setw(6) << preis/100.0 << " Euro" << endl;
 cout << "Cash  : " << setw(6) << geld /100.0 << " Euro" << endl;
 cout << "Das Wechselgeld von  " << (geld-preis)/100.0 << " Euro wiegt  "
      << setprecision(3) << masse << " Gramm." << endl << endl;

 return 0;
}

//  ####################################################################
//              Definition der Funktionen
//  ####################################################################

void num_coins(const int& price, int coins[], const int values[], const int n)
{
 int pp = price;

 for (int i=0; i<n; ++i)
 {
   coins[i] = pp / values[i];  // number of coins with that value
   pp       = pp % values[i];  // remaining price
 }

 assert(pp==0);                // exit program if pp is wrong.(it has to be 0)

 return;
}

//
void print_coins(const int coins[], const int values[], const int n)
{
 string currency("Euro");
 int    scale(100);

 cout << endl;
 for (int i=0; i<n; ++i)
 {
   if (scale>values[i])
   {
     scale = 1;
     currency = "Cent";
   }
   cout << "coins a " << setw(2) << values[i]/scale << " " << currency << ": "
                      << setw(2) << coins[i] << endl;
 }
 cout << endl;

 return;
}

//
float weight_coins(const int coins[], const float weights[], const int n)
{
 float mass = 0.0;
 for (int i=0; i<n; ++i)
 {
   mass += coins[i]*weights[i];
 }

 return mass;

}

