//    g++ -O3 -Wall -pedantic bsp_24_clock.cpp bsp_24_fkt.cpp
//    icpc  -O2 -ipo -fp-model fast -mcpu=pentium4 -alias-args  bsp_24_clock.cpp bsp_24_fkt.cpp
//      icpc  -prof-gen -O2 -ipo -ip bsp_24_clock.cpp bsp_24_fkt.cpp
//      icpc  -prof-use -O2 -ipo  bsp_24_clock.cpp bsp_24_fkt.cpp
//
//    valgrind -v --leak-check=yes --tool=memcheck  --log-file=bsp_24 --show-reachable=yes ./a.out
//    valgrind -v --tool=cachegrind  --log-file=bsp_24  ./a.out

#include <iostream>
#include <cmath>
#include <ctime>
#include <fstream>
using namespace std;
#include "bsp_24_fkt.hpp"

//--------------------------

//--------------------------
double horner_rev(const double x, const polynom& kk)
{
  double sum = kk.a[0];

  for (int k=1; k<=kk.n; ++k)
  {
    sum = x*sum + kk.a[k];        // x*()+a_k
  }

  return sum;
}

//---------------------

void test_suite_rev(const string& file, const double x)
{
 double hx;
 polynom pp;
 read_koeff(file, pp);

 for (int i = 0; i<=pp.n/2; ++i)
 {
   double tmp   = pp.a[i];
   pp.a[i]      = pp.a[pp.n-i];
   pp.a[pp.n-i] = tmp;
 }

 double t, t3;
 const int NN = 1000;
for (int j=0; j<NN; ++j)
    hx = horner_rev(x, pp);
 t = clock();
 for (int j=0; j<NN; ++j)
    hx = horner_rev(x, pp);
 t3 = (clock()-t)/CLOCKS_PER_SEC;
 cout << "   hx = " << hx <<  endl;
 cout << "   t3 = " << t3 <<  endl;

 delete [] pp.a;
}
//---------------------

void test_suite(const string& file, const double x)
{
 double mx,px,hx;
 polynom pp;
 read_koeff(file, pp);

 double t, t1,t2,t3;
 const int NN = 1000;

 t = clock();
 for (int j=0; j<NN; ++j)
    mx = polynom_mult(x, pp);
 t1 = (clock()-t)/CLOCKS_PER_SEC;
 t = clock();
 for (int j=0; j<NN; ++j)
   px = polynom_pow (x, pp);
 t2 = (clock()-t)/CLOCKS_PER_SEC;
 t = clock();
 for (int j=0; j<NN; ++j)
    hx = horner (x, pp);
 t3 = (clock()-t)/CLOCKS_PER_SEC;
 cout << " mx = " << mx << "   px = " << px << "   hx = " << hx <<  endl;
 cout << " t1 = " << t1 << "   t2 = " << t2 << "   t3 = " << t3 <<  endl;

 delete [] pp.a;
}

//--------------------------

int main()
{
 test_suite_rev("data_24_c.txt", -0.945);
 test_suite("data_24_a.txt", 3.78);
 test_suite("data_24_b.txt", -1.0087);
 test_suite("data_24_c.txt", -0.945);
 test_suite_rev("data_24_c.txt", -0.945);

//  system("pause");
 return 0;
}


//       gcc ohne Optimierung (Datensatz c  in bsp_24_clock.cpp)
//   Mult           Power            Horner
// mx = 2.12677   px = 2.12677   hx = 2.12677         // Erg
// t1 = 84.22     t2 = 14.29     t3 = 1.18            // Sec

//       gcc mit Optimierung -O3  (Datensatz c  in bsp_24_clock.cpp)
// mx = 2.12677   px = 2.12677   hx = 2.12677         // Erg
// t1 = 0.38      t2 = 11.82     t3 = 0.53            // Sec

//       icc mit Optimierung -O2  (Datensatz c  in bsp_24_clock.cpp)
// mx = 2.12677   px = 2.12677   hx = 2.12677         // Erg
// t1 = 0.40      t2 = 12.75     t3 = 0.52            // Sec
