#include <iostream>
#include <fstream>
#include <cassert>
#include <string>
#include <cmath>
using namespace std;

#include "bsp_27_fkt.hpp"

//   Implementation of the methods from class polynom

//
polynom::polynom()
 : ngrad(-1), coeff(0)               // member initialization
{
//   ngrad = -1;                     // conventional initialization
//   coeff = 0;
}


//
polynom::polynom(const polynom& rhs)
{
  ngrad = rhs.ngrad;
  coeff = new double [ngrad+1];

  for (int i=0; i<=ngrad; ++i)
  {
     coeff[i] = rhs.coeff[i];
  }

  return;
}


//
polynom::polynom(const string& file_name)
 : ngrad(-1), coeff(0)            // guarantee defined values
{
 ifstream fin(file_name.c_str());

 if( fin.is_open() )
 {
    fin >> ngrad;
//     cout << ngrad << endl;
    coeff = new double [ngrad+1];   // allocate ngrad+1 !!
    for (int i=0; i<=ngrad; ++i)
    {
      fin >> coeff[i];
//     cout << coeff[i] << endl;
    }
 }
 else
 {
    cout << "\nFile " << file_name << "have not been found.\n" ;
    assert( fin.is_open() );         // exeption handling for the poor programmer
 }
 return;
}


//
polynom::~polynom()
{
  delete [] coeff;
  return;
}


//
polynom& polynom::operator=(const polynom& rhs)
{
  if ( &rhs != this)
  {
    delete [] coeff;

    ngrad = rhs.ngrad;
    coeff = new double [ngrad+1];

    for (int i=0; i<=ngrad; ++i)
    {
       coeff[i] = rhs.coeff[i];
    }
  }

  return *this;
}


ostream & operator<<(ostream  & s, const polynom & orig)
{
  s << " Polynom of degree " << orig.ngrad << "has coefficient in decending order" << endl;
  cout.precision(3);
  for (int i=0; i<=orig.ngrad; ++i)
    {
       s << orig.coeff[i] << "  ";
    }
  s << endl;

  return s;
}

//
double polynom::horner(const double x)
{
  double sum = coeff[ngrad];

  for (int k=ngrad-1; k>=0; --k)
  {
//     sum = x*sum + coeff[k];        // x*()+a_k
    sum *= x;
    sum += coeff[k];        // x*()+a_k
  }

  return sum;
}


//
double polynom::polynom_mult(const double x)
{
  double sum = coeff[0],
         xp  = 1.0;    // x^0

  for (int k=1; k<=ngrad; ++k)
  {
    xp  *= x;             // x^k
    sum += coeff[k]*xp;   // + a_k*x^k
  }

  return sum;
}


//
double polynom::polynom_pow(const double x)
{
  double sum = 0.0;
  for (int k=0; k<=ngrad; ++k)
  {
    sum += coeff[k]*pow(x,k);   // + a_k*x^k
  }

  return sum;
}

//  #########  Playing around with loop unrolling pays off for horner2 and polynom_mult2  #####
//
double polynom::horner2(const double x)
{
  const double xx = x*x;
  double sum = coeff[ngrad];

  int n = ngrad-1;
  if ( n%2 == 0)
  {
    sum = sum*x+coeff[n];
    --n;
  }

  for (int k=n; k>=0; k -= 2)
  {
//     sum = (x*sum + coeff[k])*x+coeff[k-1];
    const double t = coeff[k]*x+coeff[k-1];
    sum = xx*sum + t;
  }

  return sum;
}

double polynom::horner4(const double x)
{
  const double xx = x*x,
              xxx = xx*x,
             xxxx = xx*xx;
  double sum = coeff[ngrad];
  int n;

  for ( n=ngrad-1; n%4 != 3; --n)
  {
    sum = x*sum + coeff[n];
  }
// cout << n << " ";
  for (int k=n; k>=0; k -= 4)
  {
//     sum = (x*sum + coeff[k])*x+coeff[k-1];
    const double t = coeff[k]*xxx+coeff[k-1]*xx+coeff[k-2]*x+coeff[k-3];
    sum = xxxx*sum + t;
  }

  return sum;
}


double polynom::polynom_mult2(const double x)
{
  double sum1 = coeff[0],
         xp1  = 1.0,
         xp2  = x,
         sum2 = x*coeff[1];
  const double xx = x*x;

  int n = ngrad;
  if ( ngrad%2 == 0)  --n;

  for (int k=2; k<=n; k+=2)
  {
    xp1  *= xx;
    sum1 += coeff[k]*xp1;
    xp2  *= xx;
    sum2 += coeff[k+1]*xp2;
  }

  double sum = sum1+sum2;
  if ( ngrad%2 == 0)  sum += coeff[ngrad]*xp1*xx;

  return sum;
}
