//   kdoc -f html -d bsp29.DOCU -n polynom -p bsp_29_fkt.hpp

#ifndef BSP_29_FKT
#define BSP_29_FKT

#include <iostream>
#include <vector>
using namespace std;

/**
  A class for 1D-polynomials p(x) with real coefficients.
  We use the template class vector is basis class for our class <T>.
  @short   A class for polynomials with real coefficients.
  @author  Gundolf Haase <Gundolf.Haase@uni-graz.at>
  @since   June 1, 2007
*/

class polynom : public vector<double>
{
//        Methods
 public:
//
   polynom();
//
   polynom(const string& file_name);
//
//   ~polynom();     // destructor of  vector<double>  is used automatically
//
/**
  Calculates p(@p x) via the Horner scheme
  @param  x  evaluation point
  @return    p(@p x)
*/
   double horner(const double x) const;

/**
  Calculates p(@p x) via the a simple loop wherein the power of
  @p x is increased in each loop by multiplication.
  @param  x  evaluation point
  @return    p(@p x)
*/
   double polynom_mult(const double x) const;

/**
  Calculates p(@p x) via the a simple loop wherein the power of
  @p x is calculated with the pow(x,n) function.
  @param  x  evaluation point
  @return    p(@p x)
*/
   double polynom_pow(const double x) const;

//   Inlined access methods
/**
   @return  polynomial degree
*/
   const int degree() const
     { return size()-1; };

/**
   @return  coefficient for index @p i which is related to the i-th power of x
*/
//    const double& operator[](const int&i) const
//      { return (*this)[i]; };

/**
   @return  coefficient for index @p i which is related to the i-th power of x
*/
//    double& operator[](const int&i)
//      { return (*this)[i]; };

};

//  No friend method of class polynom anymore because of access methods
ostream & operator<<(ostream  & s, const polynom & orig);
#endif
