#include <iostream>
using namespace std;

#include "bsp_29_fkt.hpp"

//---------------------------------------------------------------------

 SparSchwein:: SparSchwein(const int max_coins,
                           const int num_1c, const int num_10c,
                           const int num_50c, const int num_1e)
 : _cent_1(num_1c), _cent_10(num_10c), _cent_50(num_50c), _euro_1(num_1e),
   _max_coins(max_coins), _broken(false)
 {
//   Too much coins already?
   if (_cent_1+_cent_10+_cent_50+_euro_1 > _max_coins )
   {
      cout << " WARNING at  " << __FILE__ << ":" << __LINE__ << endl;
      cout << " Too much coins for piggy bank.\n I have to reduce the number of coins\n";
      // How many coins are too much?
      unsigned int n2much = (_cent_1+_cent_10+_cent_50+_euro_1)-_max_coins;
      // Reduce 1-cent coints
      unsigned int nr = min(n2much, _cent_1);
      _cent_1 -= nr;
      n2much  -= nr;
      if ( n2much > 0)
      {
        // Reduce 10-cent coints
        nr = min(n2much, _cent_10);
        _cent_10 -= nr;
        n2much   -= nr;
      }
      if ( n2much > 0)
      {
        // Reduce 50-cent coints
        nr = min(n2much, _cent_50);
        _cent_50 -= nr;
        n2much   -= nr;
      }
      if ( n2much > 0)
      {
        // Reduce 1-euro coints
        nr = min(n2much, _euro_1);
        _euro_1  -= nr;
        n2much   -= nr;
      }
   }
   return;
 }
//---------------------------------------------------------------------

 void SparSchwein::init()
 {
   _cent_1 = _cent_10 = _cent_50 = _euro_1 = 0;
   _broken = false;
   return;
 }
//---------------------------------------------------------------------
 bool SparSchwein:: isEmpty()
 {
   return (_cent_1+_cent_10+_cent_50+_euro_1 == 0);
 }

//---------------------------------------------------------------------
 bool SparSchwein:: isFull()
 {
   return (_cent_1+_cent_10+_cent_50+_euro_1 >= _max_coins);
 }

//---------------------------------------------------------------------
 bool SparSchwein:: isBroken()
 {
   return _broken;
 }

//---------------------------------------------------------------------
int SparSchwein:: add1Cents(const unsigned int n)
{
//  number of coins in bank
  const unsigned int ncoins    = _cent_1+_cent_10+_cent_50+_euro_1;
//  remaining space for coins
  const unsigned int n_allowed = min(n, _max_coins - ncoins);

  _cent_1 += n_allowed;
  return n - n_allowed;
}

//---------------------------------------------------------------------
int SparSchwein:: add10Cents(const unsigned int n)
{
//  number of coins in bank
  const unsigned int ncoins    = _cent_1+_cent_10+_cent_50+_euro_1;
//  remaining space for coins
  const unsigned int n_allowed = min(n, _max_coins - ncoins);

  _cent_10 += n_allowed;
  return n - n_allowed;
}

//---------------------------------------------------------------------
int SparSchwein:: add50Cents(const unsigned int n)
{
//  number of coins in bank
  const unsigned int ncoins    = _cent_1+_cent_10+_cent_50+_euro_1;
//  remaining space for coins
  const unsigned int n_allowed = min(n, _max_coins - ncoins);

  _cent_50 += n_allowed;
  return n - n_allowed;
}

//---------------------------------------------------------------------
int SparSchwein:: add1Euro(const unsigned int n)
{
//  number of coins in bank
  const unsigned int ncoins    = _cent_1+_cent_10+_cent_50+_euro_1;
//  remaining space for coins
  const unsigned int n_allowed = min(n, _max_coins - ncoins);

  _euro_1 += n_allowed;
  return n - n_allowed;
}

//---------------------------------------------------------------------
 float SparSchwein:: breakInto()
 {
  //  money in bank
  float tmp = _cent_1*_val_1c+_cent_10*_val_10c+_cent_50*_val_50c+_euro_1*_val_1e;

  init();   // reset everything in piggy bank

  return  tmp;
 }

//---------------------------------------------------------------------


//---------------------------------------------------------------------

ostream& operator<<(ostream& s, const SparSchwein& orig)
{
  s << endl;
  s << "Piggy bank: "
    << orig._cent_1*orig._val_1c+orig._cent_10*orig._val_10c
      +orig._cent_50*orig._val_50c+orig._euro_1*orig._val_1e
    << " cents" << endl;
  s << "  1-cent coins: " << orig._cent_1  << endl;
  s << " 10-cent coins: " << orig._cent_10 << endl;
  s << " 50-cent coins: " << orig._cent_50 << endl;
  s << "  1-euro coins: " << orig._euro_1  << endl;
  s << endl;
 return s;
}
